#include <stdint.h>
#include "built_in.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOC_PDOR;
sbit TFT_RST at GPIOC_PDOR.B12;
sbit TFT_RS at GPIOC_PDOR.B14;
sbit TFT_CS at GPIOC_PDOR.B13;
sbit TFT_RD at GPIOC_PDOR.B15;
sbit TFT_WR at GPIOC_PDOR.B16;
sbit TFT_BLED at GPIOB_PDOR.B18;
// End TFT module connections

unsigned char adc_conv_res[12];
uint16_t ADC_Result = 0;

void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _hi;
  tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _lo;
}

void Write_to_Data_Lines_8bit(unsigned char _data){
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _data;
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  volatile unsigned short temp;
  TFT_RS = 1;
  temp = (_data>>11);
  temp = (temp<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data>>5);
  temp = (temp<<2);
  if ((temp>>7) == 1) {
    temp += 3;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
}

static void InitDisplay() {
  TFT_Set_Active(TFT_SSD1963_8bit_Set_Index,TFT_SSD1963YT_8bit_Write_Command,TFT_SSD1963_8bit_Write_Data);
  TFT_Init_SSD1963_Board_35(320, 240);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Font(TFT_defaultFont,CL_BLACK,FO_HORIZONTAL);
  TFT_Write_Text("MM3 ADC KINETIS TEST",100,25);
  TFT_Write_Text("--------------------",100,40);
}

void main() {

   InitDisplay();
   ADC0_Init();                                                // Init ADC1
   ADC0_Set_Input_Channel(_ADC_CHANNEL_13);                    // Choose ADC channel 12 for PTB6
   Delay_ms(100);
   
     while(1) {
       ADC_Result = ADC0_Get_Sample(13);
       WordToStr(ADC_Result,adc_conv_res);
         TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
         TFT_Set_Pen(CL_WHITE,1);
         TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
         TFT_Rectangle(85,100,200,190);
         TFT_Write_Text(adc_conv_res, 85, 100);
       Delay_ms(50);
   }
}