#include <stdint.h>
#include "built_in.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOC_PDOR;
sbit TFT_RST at GPIOC_PDOR.B12;
sbit TFT_RS at GPIOC_PDOR.B14;
sbit TFT_CS at GPIOC_PDOR.B13;
sbit TFT_RD at GPIOC_PDOR.B15;
sbit TFT_WR at GPIOC_PDOR.B16;
sbit TFT_BLED at GPIOB_PDOR.B18;
// End TFT module connections

#define ARRAY_LENGHT 50

char someData[ARRAY_LENGHT];
char data_[10];
char rData[ARRAY_LENGHT];
char i;


void EEPROM_24C02_WrSingle(unsigned short wAddr, unsigned short wData) {
  data_[0] = wAddr;
  data_[1] = wData;
  I2C_Start();
  // issue I2C start signal
  I2C_Write(0x50,data_,2,END_MODE_STOP);
}//~

//--------------- Reads data from 24C02 EEPROM - single location (random)
unsigned short EEPROM_24C02_RdSingle(unsigned short rAddr) {
  data_[0] = rAddr;
  I2C_Start();              // issue I2C start signal
  I2C_Write(0x50,data_,1,END_MODE_RESTART);
  I2C_Read(0x50,data_,1,END_MODE_STOP);

  return data_[0];
}

void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _hi;
  tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _lo;
}

void Write_to_Data_Lines_8bit(unsigned char _data){
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _data;
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  volatile unsigned short temp;
  TFT_RS = 1;
  temp = (_data>>11);
  temp = (temp<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data>>5);
  temp = (temp<<2);
  if ((temp>>7) == 1) {
    temp += 3;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
}

static void InitDisplay() {
  TFT_Set_Active(TFT_SSD1963_8bit_Set_Index,TFT_SSD1963YT_8bit_Write_Command,TFT_SSD1963_8bit_Write_Data);
  TFT_Init_SSD1963_Board_35(320, 240);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Write_Text("MM3 I2C KINETIS TEST",100,25);
  TFT_Write_Text("--------------------",100,40);

}

void SystemInit()
{
  InitDisplay();
}
int j,k = 50;
void main() {
  short i = 0;
  short flag = 0;
  unsigned short testArray[10] = {0};
  char string[10] = {0};
  SystemInit();
  I2C1_Init_Advanced(100000, &_GPIO_Module_I2C1_PC10_11);        // performs I2C initialization

  memset(someData, 0x00, sizeof(someData));
  memset(rData, 0x01, sizeof(someData));               // something not zero

  Delay_ms(100);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Pen(CL_WHITE,1);
  TFT_Rectangle(85,100,200,190);
  TFT_Write_Text("Erasing :", 55, 60);

  for(i = 0; i < ARRAY_LENGHT; i++) {
    EEPROM_24C02_WrSingle(i,0x00);
    TFT_Write_Text("*",k,150);
    k = k+2;
    Delay_ms(5);
  }
  TFT_Write_Text("100%",170,145);

  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    Delay_ms(10);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
    TFT_Write_Text("Done!", 120, 60);
    Delay_ms(100);
  }

  TFT_Write_Text("Writting :", 55, 80);

  for(i = 0; i < sizeof(someData); i++) {
    EEPROM_24C02_WrSingle(i,i);
    someData[i] = i;
    Delay_ms(5);
  }

  TFT_Write_Text("Complete", 120, 80);
  TFT_Write_Text("Reading :", 55, 100);

  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    Delay_ms(10);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
    TFT_Write_Text("Success - Full match!", 120, 100);
  }
  else {
    TFT_Write_Text("Failed", 120, 100);
  }

  while(1)
    ;
}