#include <stdint.h>
#include "built_in.h"

// TFT module connections
unsigned int TFT_DataPort at GPIOC_PDOR;
sbit TFT_RST at GPIOC_PDOR.B12;
sbit TFT_RS at GPIOC_PDOR.B14;
sbit TFT_CS at GPIOC_PDOR.B13;
sbit TFT_RD at GPIOC_PDOR.B15;
sbit TFT_WR at GPIOC_PDOR.B16;
sbit TFT_BLED at GPIOB_PDOR.B18;
// End TFT module connections

const short _C3DHALL7_REG_DEVICE_ID = 0;

sbit SPI_CS at GPIOD_PDOR.B11;


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _hi;
  tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _lo;
}

void Write_to_Data_Lines_8bit(unsigned char _data){
  unsigned int tmp = GPIOC_PDOR;
  tmp &= 0xFF00;
  GPIOC_PDOR = tmp | _data;
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  volatile unsigned short temp;
  TFT_RS = 1;
  temp = (_data>>11);
  temp = (temp<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data>>5);
  temp = (temp<<2);
  if ((temp>>7) == 1) {
    temp += 3;
  }
  Write_to_Data_Lines_8bit(temp);
  temp = (_data<<3);
  if ((temp>>7) == 1) {
    temp += 7;
  }
  Write_to_Data_Lines_8bit(temp);
}

static void InitDisplay() {
  TFT_Set_Active(TFT_SSD1963_8bit_Set_Index,TFT_SSD1963YT_8bit_Write_Command,TFT_SSD1963_8bit_Write_Data);
  TFT_Init_SSD1963_Board_35(320, 240);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Write_Text("MM3 FPI KINETIS TEST",100,25);
  TFT_Write_Text("--------------------",100,40);

}


typedef struct
{
    uint8_t company_id;
    uint8_t device_id;

}T_C3DHALL7_DEV_INFO;

T_C3DHALL7_DEV_INFO info;


static void hal_spiWrite(uint8_t *pBuf, uint16_t nBytes)
{
    uint8_t *ptr = pBuf;
    while( nBytes-- )
        SPI_Write( *( ptr++ ) );
}

static void hal_spiRead(uint8_t *pBuf, uint16_t nBytes)
{
    uint8_t *ptr = pBuf;
    while( nBytes-- )
        *( ptr++ ) = SPI_Read( 0x00 );
}

void c3dhall7_readData(uint8_t reg, uint8_t *dataOut, uint8_t nBytes){
  uint8_t writeReg[ 1 ];

  writeReg[ 0 ] = reg | 0x80;

  SPI_CS = 0;
  hal_spiWrite( writeReg, 1 );
  hal_spiRead( dataOut, nBytes );
  SPI_CS = 1;

}

void c3dhall7_deviceInfo(T_C3DHALL7_DEV_INFO *info)
{
    uint8_t readData[ 2 ];

    c3dhall7_readData(_C3DHALL7_REG_DEVICE_ID, readData, 2);

    info->company_id = readData[ 0 ];
    info->device_id = readData[ 1 ];
}

void SystemInit()
{
  GPIO_Digital_Output(&GPIOD_PDOR, _GPIO_PINMASK_11);
  SPI_CS = 1;
  Delay_100ms();

  SPI2_Init_Advanced(8000000, _SPI_CFG_MASTER |
                              _SPI_CFG_FRAME_SIZE_8BITS |
                              _SPI_CFG_CLK_IDLE_LOW |
                              _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                              _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI2_PD12_14_13);
  Delay_100ms();
  InitDisplay();
}

void main() {
  short i = 0;
  short flag = 0;
  unsigned short testArray[10] = {0};
  char string[10] = {0};
  
  
  SystemInit();
  
  for(i = 0; i < 10; i++)
  {
    c3dhall7_deviceInfo(&info);
    testArray[i] = info.device_id;
    Delay_ms(200);
  }
  for(i = 0; i < 10; i++)
  {
    if(testArray[i] != 192)
      flag = 1;
  }
  if(flag)
  TFT_Write_Text("SPI Test failed",50,50);
  else
  TFT_Write_Text("SPI Test OK",100,100);

  while(1)
    ;
}