/*
    mmb_---.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   mmb_---.h

- Version               :   1.0.0
- Date                  :   May 2019
- Developer             :   Milos Vidojevic

    Generated from mmb---.h template.

    ---

    Template for definition of MCU connections used on mikromedia systems. This 
    file should be used as start point for board definition file development. 

    Hints :

    - Copy file inside [project base directory]/portable.
    - Replace all "---" appearances with proper values.

*/
/* -------------------------------------------------------------------------- */

#include "compiler.h"

#ifndef _MMB_3_KINETIS_
#define _MMB_3_KINETIS_

/*
    REF SEL jumper position :

    3V3         - 1
    2.048V      - 0
*/

#define JP_VREF                     0

/*
    MP3 jumper position :

    LIN         - 1
    MIC         - 0
*/

#define JP_MP3                      0

/*
    ACCEL jumpers position.
*/

#define JP_ACCEL_A                  0
#define JP_ACCEL_B                  0

/*
    TFT jumpers position.
*/

#define JP_TFT_LR                   1
#define JP_TFT_UD                   0
#define JP_TFT_DITH                 1
#define JP_TFT_MODE                 0

// ============================================================================
//                                                                    DISPLAY
// ============================================================================

extern sfr sbit TFT_BLED;
extern sfr sbit TFT_CS;
extern sfr sbit TFT_RD;
extern sfr sbit TFT_RS;
extern sfr sbit TFT_RST;
extern sfr sbit TFT_WR;

static void Write_to_Data_Lines( unsigned char _hi, unsigned char _lo )
{
    unsigned int tmp = GPIOC_PDOR;
    tmp &= 0xFF00;
    GPIOC_PDOR = tmp | _hi;
    tmp = GPIOC_PDOR;
    tmp &= 0xFF00;
    GPIOC_PDOR = tmp | _lo;
}

static void Write_to_Data_Lines_8bit( unsigned char _data )
{
    unsigned int tmp = GPIOC_PDOR;
    tmp &= 0xFF00;
    GPIOC_PDOR = tmp | _data;
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Set_Index( unsigned short index )
{
    TFT_RS = 0;
    Write_to_Data_Lines( 0, index );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Command( unsigned short cmd )
{
    TFT_RS = 1;
    Write_to_Data_Lines( 0, cmd );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Data( unsigned int _data )
{
    volatile unsigned short temp;
    
    TFT_RS = 1;
    temp = ( _data >> 11 );
    temp = ( temp << 3 );
    
    if (( temp >> 7 ) == 1)
    {
      temp += 7;
    }
    
    Write_to_Data_Lines_8bit( temp );
    temp = ( _data >> 5 );
    temp = ( temp << 2 );
    
    if (( temp >> 7 ) == 1)
    {
        temp += 3;
    }
    
    Write_to_Data_Lines_8bit( temp );
    temp = (_data << 3);
    
    if (( temp >> 7 ) == 1)
    {
        temp += 7;
    }
    
    Write_to_Data_Lines_8bit( temp );
}

static void hal_display_init ( void )
{
    //  Initialize display.

    TFT_Set_Active( TFT_SSD1963_8bit_Set_Index, TFT_SSD1963YT_8bit_Write_Command, TFT_SSD1963_8bit_Write_Data );
    TFT_Init_SSD1963_Board_35( 320, 240 );
    TFT_Set_DBC_SSD1963(255);
}

// ============================================================================
//                                                                      TOUCH
// ============================================================================

static uint8_t hal_touch_init ( void )
{
//  Initialize touch senor.

    GPIO_Digital_Output( &GPIOB_PDOR, _GPIO_PINMASK_0 );
    GPIO_Digital_Input( &GPIOB_PDIR, _GPIO_PINMASK_1 );

    GPIOB_PDOR.B0 = 1;

    while (GPIOB_PDIR.B1 == 0)
    {
        GPIOB_PDOR.B0 = 0;
        Delay_us(10);
        GPIOB_PDOR.B0 = 1;
        Delay_us(10);
    }

    I2C0_Init_Advanced( 400000, &_GPIO_Module_I2C0_PB0_1 );

    FT5XX6_SetI2CAddress(0x38);
    
    if (FT5XX6_IsOperational( ) != FT5XX6_OK)
    {
        return  FT5XX6_FAILURE;
    }

    FT5XX6_SetDefaultMode( );
    FT5XX6_SetController( _TC_FT5X46 );
    FT5XX6_SetSizeAndRotation( 320, 240, 2 );

    return FT5XX6_OK;
}

// ============================================================================
//                                                                    CONSOLE
// ============================================================================

//  Map proper functions used with console.

#define CONSOLE_WRITE_TXT             UART0_Write_Text
#define CONSOLE_READY                 UART0_Data_Ready
#define CONSOLE_READ                  UART0_Read

static void hal_console_init ( void )
{
    //  Initialize console.

    UART0_Init_Advanced(
                         115200,
                         _UART_8_BIT_DATA,
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT,
                         &_GPIO_Module_UART5_PD9_8
                        );
}

// ============================================================================
//                                                                      POWER
// ============================================================================

#if ( _MMB_MODULE_POWER == 1 )

//  Detects charging of the battery.

static sbit BSP_CHARGER_STATUS at           GPIOB_PDIR.B12;

#if ( JP_VREF == 0 )
#   define BSP_POWER_ADC_VREF               ( 2.048f )
#elif ( JP_VREF == 1 )
#   define BSP_POWER_ADC_VREF               ( 3.300f )
#else
#   error "REF SEL POSITION UNKNOWN"
#endif

#define BSP_POWER_ADC_CHANNEL               ( 17 )
#define BSP_POWER_ADC_RESOLUTION            ( 4096.0f )

const static hal_adc_obj_t BSP_POWER_ADC_MODULE =
{
    ADC0_Read,
    ADC0_Get_Sample
};

static void hal_power_init ( void )
{
    //  Initialize BSP_CHARGER_STATUS as input pin

    GPIO_Digital_Input( &GPIOB_PDIR, _GPIO_PINMASK_12 );

    //  Initialize ADC module

    SIM_SCGC6 = 0x8000000;
    GPIO_Analog_Input( &GPIOE_PDIR, _GPIO_PINMASK_24 );
    Delay_100ms( );

    ADC0_Init( );
}

#endif

// ============================================================================
//                                                                    MICROSD
// ============================================================================

#if ( _MMB_MODULE_MICROSD == 1 )

static void hal_microsd_init ( void )
{
    Mmc_Set_Interface( _MMC_INTERFACE_SDHC );
}

#endif

// ============================================================================
//                                                                      FLASH
// ============================================================================

#if ( _MMB_MODULE_FLASH == 1 )

static sbit BSP_FLASH_CE at                 GPIOD_PDOR.B10;

const static hal_spi_obj_t BSP_FLASH_SPI_MODULE =
{
    SPI0_Write,
    SPI0_Read
};

static void hal_flash_init ( void )
{
    //  Initialize BSP_FLASH_CE as output pin.
    
    GPIO_Digital_Output( &GPIOD_PDOR, _GPIO_PINMASK_10 );

    //  Initialize SPI module.
    
    SPI0_Init_Advanced( 1000000,
                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,
                        &_GPIO_Module_SPI0_PD1_3_2 );                        // _GPIO_Module_SPI1_PB11_17_16

    BSP_FLASH_CE = 1;
}

#endif

// ============================================================================
//                                                                        MP3
// ============================================================================

#if ( _MMB_MODULE_MP3 == 1 )

static volatile sbit BSP_MP3_DREQ at                 GPIOA_PDIR.B16;
static volatile sbit BSP_MP3_CS at                   GPIOA_PDOR.B14;
static volatile sbit BSP_MP3_RST at                  GPIOA_PDOR.B15;
static volatile sbit BSP_MP3_DCS at                  GPIOA_PDOR.B17;

static const hal_spi_obj_t BSP_MP3_SPI_MODULE =
{
    SPI0_Write,
    SPI0_Read
};

static void hal_mp3_init ( void )
{
    //  Initialize BSP_MP3_DREQ as input pin.

    GPIO_Digital_Input( &PTA_PDIR, _GPIO_PINMASK_16 );

    //  Initialize BSP_MP3_CS as output pin.

    GPIO_Digital_Output( &PTA_PDOR, _GPIO_PINMASK_14 );

    //  Initialize BSP_MP3_RST as output pin.

    GPIO_Digital_Output( &PTA_PDOR, _GPIO_PINMASK_15 );

    //  Initialize BSP_MP3_DCS as output pin.

    GPIO_Digital_Output( &PTA_PDOR, _GPIO_PINMASK_17 );

    //  Initialize SPI module.

    // Initialize SPI2 module
    SPI0_Init_Advanced( 1000000,
                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,
                        &_GPIO_Module_SPI0_PD1_3_2 );
}

#endif

// ============================================================================
//                                                                      ACCEL
// ============================================================================

#if ( _MMB_MODULE_ACCEL == 1 )

static sbit BSP_ACCEL_INT_1 at                          GPIOC_PDIR.B18;
static sbit BSP_ACCEL_INT_2 at                          GPIOC_PDIR.B19;

const static hal_i2c_obj_t BSP_ACCEL_I2C_MODULE =
{
    I2C0_Start,
    I2C0_Write,
    I2C0_Read
};

static void hal_accel_init ( void )
{
    //  Initialize BSP_ACCEL_INT_1 as input pin.
    
    GPIO_Digital_Input( &PTC_PDIR, _GPIO_PINMASK_18 );
    
    //  Initialize BSP_ACCEL_INT_2 as input pin.
    
    GPIO_Digital_Input( &PTC_PDIR, _GPIO_PINMASK_19 );

    //  Initialize I2C module. 

    I2C0_Init_Advanced( 100000, &_GPIO_Module_I2C0_PB0_1 );
}

#endif

// ============================================================================
//                                                                        USB
// ============================================================================

#if ( _MMB_MODULE_USB == 1 )

#ifndef __KINETIS__

//  Enables power for USB device.

static sbit BSP_USB_PSW at                          ---;

static void hal_usb_init ( void )
{
    //  Enable alternate GPIO function if needed

    ---

    //  Initialize PWS pin as output.

    ---

    // PSW to 0 to turn on power for device.

    BSP_USB_PSW = 1;
    Delay_ms(100);
    BSP_USB_PSW = 0;
}

#endif

#endif

// ============================================================================
//                                                                    SHUTTLE
// ============================================================================

#if ( _MMB_MODULE_SHUTTLE == 1 )



#endif

#endif

/* -------------------------------------------------------------------------- */
/*
    mmb_mcu_---.h

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */